==> 0.3-1.zip: ldaptool/src/ldaptool/_main.py <==
from __future__ import annotations

import argparse
import csv
import dataclasses
import enum
import html
import subprocess
import sys
import typing

from ldaptool import decode, search
from ldaptool._utils import argclasses
from ldaptool._utils.ldap import Result, SizeLimitExceeded


class TableOutput(enum.StrEnum):
    MARKDOWN = "markdown"
    CSV = "csv"
    HTML = "html"


def _html_escape_line(columns: typing.Sequence[str], *, cell: str = "td") -> str:
    cell_s = f"<{cell}>"
    cell_e = f"</{cell}>"
    return "<tr>" + ("".join(cell_s + html.escape(col) + cell_e for col in columns)) + "</tr>\n"


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(search.Arguments):
    raw: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Don't pipe output through ldap-decode"),
    )
    csv: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="CSV output - requires list of attributes"),
    )
    table: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Markdown table output - requires list of attributes",
        ),
    )
    table_output: typing.Optional[TableOutput] = None
    html: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="HTML table output - requires list of attributes",
        ),
    )
    sort: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Sorted table output - defaults to markdown --table unless --csv is given",
        ),
    )

    def __post_init__(self) -> None:
        super(Arguments, self).__post_init__()  # super() not working here, unclear why.

        # pick at most one in csv, (markdown) table, html
        if [self.csv, self.table, self.html].count(True) > 1:
            raise SystemExit("Can't use more than one table output type")

        if self.csv:
            self.table_output = TableOutput.CSV
        elif self.table:
            self.table_output = TableOutput.MARKDOWN
        elif self.html:
            self.table_output = TableOutput.HTML

        if self.sort and self.table_output is None:
            # default to markdown table
            self.table_output = TableOutput.MARKDOWN

        if self.table_output:
            if not self.columns:
                raise SystemExit("Table output requires attributes")
            if self.json:
                raise SystemExit("Can't use both table output and --json")
            if self.human:
                raise SystemExit("Can't use both table output and --human")

        if self.raw:
            if self.table_output:
                raise SystemExit("Table output requires decode; --raw not allowed")
            if self.json or self.human:
                raise SystemExit("Decode options require decode; --raw not allowed")


class _Context:
    def __init__(self) -> None:
        parser = argparse.ArgumentParser()
        arguments_p = Arguments.add_to_parser(parser)
        args = parser.parse_args()
        try:
            self.config = search.Config.load()
        except Exception as e:
            raise SystemExit(f"config error: {e}")
        self.arguments = arguments_p.from_args(args)

    def run(self) -> None:
        # starting the search sets the base we want to print
        search_iterator = search.search(config=self.config, arguments=self.arguments)
        self._run_with_filters(search_iterator)

    def _run_with_filters(self, search_iterator: typing.Iterable[Result]) -> None:
        output: typing.IO[str] = sys.stdout
        procs: list[subprocess.Popen[str]] = []

        def add_filter(cmd: list[str]) -> None:
            nonlocal output
            proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=output, text=True)
            procs.append(proc)
            if output != sys.stdout:
                output.close()
            assert proc.stdin
            output = proc.stdin

        try:
            if self.arguments.table_output == TableOutput.MARKDOWN:
                add_filter(["csvlook"])
            if self.arguments.table_output:
                self._table_output(search_iterator, stream=output)
            else:
                self._ldif_or_json_output(search_iterator, stream=output)
        finally:
            if procs:
                output.close()
            for proc in reversed(procs):
                proc.wait()

    def _to_table_lines(self, search_iterator: typing.Iterable[Result]) -> typing.Iterable[tuple[str, ...]]:
        decoder = decode.Decoder(arguments=self.arguments)
        # "human" (json) dicts contain data by lower case key:
        column_keys = [col.lower() for col in self.arguments.columns]
        try:
            for dn, entry in search_iterator:
                if dn is None:
                    continue
                # normal entry
                assert not isinstance(entry, list)
                obj = decoder.human(dn=dn, entry=decoder.read(dn=dn, entry=entry))
                yield tuple(obj.get(key, "") for key in column_keys)
        except SizeLimitExceeded as e:
            raise SystemExit(f"Error: {e}")

    def _table_output(self, search_iterator: typing.Iterable[Result], *, stream: typing.IO[str]) -> None:
        line_iterator = self._to_table_lines(search_iterator)
        if self.arguments.sort:
            line_iterator = sorted(line_iterator)

        if self.arguments.table_output in [TableOutput.CSV, TableOutput.MARKDOWN]:
            csv_out = csv.writer(stream, lineterminator="\n")
            csv_out.writerow(self.arguments.columns)

            for line in line_iterator:
                csv_out.writerow(line)
        else:
            assert self.arguments.table_output == TableOutput.HTML

            stream.write("<table>\n")
            stream.write(_html_escape_line(self.arguments.columns, cell="th"))
            for line in line_iterator:
                stream.write(_html_escape_line(line))
            stream.write("</table>\n")

    def _ldif_or_json_output(self, search_iterator: typing.Iterable[Result], *, stream: typing.IO[str]) -> None:
        decoder = decode.Decoder(arguments=self.arguments)

        num_responses = 0
        num_entries = 0

        ldif_output = not (self.arguments.json or self.arguments.human)

        if ldif_output:
            print("# extended LDIF")
            print("#")
            print("# LDAPv3")
            print(f"# base <{self.arguments.base}> with scope subtree")
            print(f"# filter: {self.arguments.filter}")
            if self.arguments.attributes:
                print(f"# requesting: {' '.join(self.arguments.attributes)}")
            else:
                print("# requesting: ALL")
            print("#")
            print()

        try:
            for dn, entry in search_iterator:
                num_responses += 1
                if dn is None:
                    if ldif_output:
                        print("# search reference")
                        for ref in entry:
                            assert isinstance(ref, str)
                            print(f"ref: {ref}")
                        print()
                    continue
                # normal entry
                assert not isinstance(entry, list)
                num_entries += 1
                obj = decoder.read(dn=dn, entry=entry)
                decoder.emit(dn=dn, entry=obj)
        except SizeLimitExceeded as e:
            raise SystemExit(f"Error: {e}")

        if ldif_output:
            print(f"# numResponses: {num_responses}")
            print(f"# numEntries: {num_entries}")


def main() -> None:
    ctx = _Context()
    ctx.run()

==> 0.4-1.zip: ldaptool/src/ldaptool/_main.py <==
from __future__ import annotations

import argparse
import csv
import dataclasses
import enum
import html
import subprocess
import sys
import typing

from ldaptool import decode, search
from ldaptool._utils import argclasses
from ldaptool._utils.ldap import Result, SizeLimitExceeded


class TableOutput(enum.Enum):
    MARKDOWN = "markdown"
    CSV = "csv"
    HTML = "html"


def _html_escape_line(columns: typing.Sequence[str], *, cell: str = "td") -> str:
    cell_s = f"<{cell}>"
    cell_e = f"</{cell}>"
    return "<tr>" + ("".join(cell_s + html.escape(col) + cell_e for col in columns)) + "</tr>\n"


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(search.Arguments):
    raw: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Don't pipe output through ldap-decode"),
    )
    csv: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="CSV output - requires list of attributes"),
    )
    table: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Markdown table output - requires list of attributes",
        ),
    )
    html: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="HTML table output - requires list of attributes",
        ),
    )
    table_output: typing.Optional[TableOutput] = None
    sort: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Sorted table output - defaults to markdown --table unless --csv is given",
        ),
    )
    json: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Use full json output"),
    )
    human: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Use simple json output (join multiple values of one attribute)"),
    )

    def __post_init__(self) -> None:
        super(Arguments, self).__post_init__()  # super() not working here, unclear why.

        # pick at most one in csv, (markdown) table, html
        if [self.csv, self.table, self.html].count(True) > 1:
            raise SystemExit("Can't use more than one table output type")

        if self.csv:
            self.table_output = TableOutput.CSV
        elif self.table:
            self.table_output = TableOutput.MARKDOWN
        elif self.html:
            self.table_output = TableOutput.HTML

        if self.sort and self.table_output is None:
            # default to markdown table
            self.table_output = TableOutput.MARKDOWN

        if self.table_output:
            if not self.columns:
                raise SystemExit("Table output requires attributes")
            if self.json:
                raise SystemExit("Can't use both table output and --json")
            if self.human:
                raise SystemExit("Can't use both table output and --human")

        if self.raw:
            if self.table_output:
                raise SystemExit("Table output requires decode; --raw not allowed")
            if self.json or self.human:
                raise SystemExit("Decode options require decode; --raw not allowed")


class _Context:
    def __init__(self) -> None:
        parser = argparse.ArgumentParser()
        arguments_p = Arguments.add_to_parser(parser)
        args = parser.parse_args()
        try:
            self.config = search.Config.load()
        except Exception as e:
            raise SystemExit(f"config error: {e}")
        try:
            self.arguments = arguments_p.from_args(args)
        except decode.InvalidStep as e:
            raise SystemExit(f"invalid arguments: {e}")

    def run(self) -> None:
        # starting the search sets the base we want to print
        search_iterator = search.search(config=self.config, arguments=self.arguments)
        self._run_with_filters(search_iterator)

    def _run_with_filters(self, search_iterator: typing.Iterable[Result]) -> None:
        output: typing.IO[str] = sys.stdout
        procs: list[subprocess.Popen[str]] = []

        def add_filter(cmd: list[str]) -> None:
            nonlocal output
            proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=output, text=True)
            procs.append(proc)
            if output != sys.stdout:
                output.close()
            assert proc.stdin
            output = proc.stdin

        try:
            if self.arguments.table_output == TableOutput.MARKDOWN:
                add_filter(["csvlook"])
            if self.arguments.table_output:
                self._table_output(search_iterator, stream=output)
            else:
                self._ldif_or_json_output(search_iterator, stream=output)
        finally:
            if procs:
                output.close()
            for proc in reversed(procs):
                proc.wait()

    def _to_table_lines(self, search_iterator: typing.Iterable[Result]) -> typing.Iterable[tuple[str, ...]]:
        decoder = decode.Decoder(arguments=self.arguments)
        # "human" (json) dicts contain data by lower case key:
        column_keys = [col.lower() for col in self.arguments.columns]
        try:
            for dn, entry in search_iterator:
                if dn is None:
                    continue
                # normal entry
                assert not isinstance(entry, list)
                obj = decoder.human(dn=dn, obj=decoder.read(dn=dn, entry=entry))
                yield tuple(obj.get(key, "") for key in column_keys)
        except SizeLimitExceeded as e:
            raise SystemExit(f"Error: {e}")

    def _table_output(self, search_iterator: typing.Iterable[Result], *, stream: typing.IO[str]) -> None:
        line_iterator = self._to_table_lines(search_iterator)
        if self.arguments.sort:
            line_iterator = sorted(line_iterator)

        if self.arguments.table_output in [TableOutput.CSV, TableOutput.MARKDOWN]:
            csv_out = csv.writer(stream, lineterminator="\n")
            csv_out.writerow(self.arguments.columns)

            for line in line_iterator:
                csv_out.writerow(line)
        else:
            assert self.arguments.table_output == TableOutput.HTML

            stream.write("<table>\n")
            stream.write(_html_escape_line(self.arguments.columns, cell="th"))
            for line in line_iterator:
                stream.write(_html_escape_line(line))
            stream.write("</table>\n")

    def _ldif_or_json_output(self, search_iterator: typing.Iterable[Result], *, stream: typing.IO[str]) -> None:
        decoder = decode.Decoder(arguments=self.arguments)

        num_responses = 0
        num_entries = 0

        ldif_output = not (self.arguments.json or self.arguments.human)

        if ldif_output:
            print("# extended LDIF")
            print("#")
            print("# LDAPv3")
            print(f"# base <{self.arguments.base}> with scope subtree")
            print(f"# filter: {self.arguments.filter}")
            if self.arguments.attributes:
                print(f"# requesting: {' '.join(self.arguments.attributes)}")
            else:
                print("# requesting: ALL")
            print("#")
            print()

        try:
            for dn, entry in search_iterator:
                num_responses += 1
                if dn is None:
                    if ldif_output:
                        print("# search reference")
                        for ref in entry:
                            assert isinstance(ref, str)
                            print(f"ref: {ref}")
                        print()
                    continue
                # normal entry
                assert not isinstance(entry, list)
                num_entries += 1
                if ldif_output:
                    decoder.read_and_emit_ldif(dn=dn, entry=entry, file=stream)
                elif self.arguments.human:
                    decoder.read_and_emit_human(dn=dn, entry=entry, file=stream)
                else:
                    assert self.arguments.json
                    decoder.read_and_emit_json(dn=dn, entry=entry, file=stream)
        except SizeLimitExceeded as e:
            raise SystemExit(f"Error: {e}")

        if ldif_output:
            print(f"# numResponses: {num_responses}")
            print(f"# numEntries: {num_entries}")


def main() -> None:
    ctx = _Context()
    ctx.run()
